/* s_pq.h (priority queues)

   Ambros Marzetta

   94-10-21

   */

#ifndef _Z_PQ_H_
#define _Z_PQ_H_


/* Types ------------------------------------------------------------- */

typedef enum {z_pq_sortedlist, z_pq_heap, z_pq_numtypes} z_pq_typeT;
				/* different implementations */
#define z_pq_default z_pq_heap	/* default implementation */

typedef struct z_pqT *z_pqP;	/* priority queue */

#ifndef _Z_PQ_C_
typedef void *z_pq_elemP;	/* element of priority queue */
#else
typedef struct z_pq_elemT *z_pq_elemP;	/* element of priority queue */
#endif

typedef struct z_pq_refT *z_pq_refP; /* reference into priority queue */
typedef signed int (*z_pq_CompProcP)(z_pq_elemP x, z_pq_elemP y);
				/* < 0 if x < y */
				/* = 0 if x == y */
				/* > 0 if x > y */
typedef void (*whatToDoProcP)(z_pq_elemP);


/* Procedures -------------------------------------------------------- */

z_pqP z_pq_NewPriorityQueue(z_pq_typeT type, z_pq_CompProcP compare);
/* Allocate an empty priority queue. Choose an implementation
   of type type. */

void z_pq_FreePriorityQueue(z_pqP pq);
/* Deallocate the priority queue pq, which must be empty. */

z_pq_elemP z_pq_GetObject(z_pqP pq, z_pq_refP where);
/* Retrieve the object referenced by where. */

z_pq_refP z_pq_Insert(z_pqP pq, z_pq_elemP x);
/* Insert x into the priority queue. */

int z_pq_isEmpty(z_pqP pq);
/* Test whether the queue is empty. */

long z_pq_numElements(z_pqP pq);
/* Return the number of elements in pq. */

z_pq_refP z_pq_Minimum(z_pqP pq);
/* Return a reference to the minimum without deleting it. The queue
   must not be empty. */

void z_pq_Delete(z_pqP pq, z_pq_refP ref);
/* Delete the element referenced by ref. */

void z_pq_DeleteAbove(z_pqP pq, z_pq_elemP upperBound, whatToDoProcP whatToDo);
/* Remove all elements >= upperBound from the priority queue. Call
   whatToDo for each of them. WhatToDo must not access the priority
   queue, but should e.g. deallocate the elements. */

void z_pq_ForAll(z_pqP pq, whatToDoProcP whatToDo);
/* Call whatToDo for all elements of the priority queue in an unspecified
   order. WhatToDo must not access the priority queue. */

char *z_pq_InfoString(z_pqP pq);
/* Return statistical information about the queue. This is a static string. */

/* If the preprocessor macro RETURNNULL is defined, z_pq_NewPriorityQueue
   and z_pq_Insert return NULL if there is insufficient space. If it isn't
   defined and there is insufficient space, the program terminates.

   Restrictions for the Heap implementation:
   References become invalid when the queue is manipulated, i.e. you
   can delete only the minimum element or the element just inserted.
   */

#endif /* _Z_PQ_H_ */

